<?php

namespace Drupal\pf_alerta\Plugin\PushFrameworkChannel;

use Drupal;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\push_framework\ChannelBase;
use Drupal\user\UserInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;

/**
 * Plugin implementation of the push framework channel.
 *
 * @ChannelPlugin(
 *   id = "alerta",
 *   label = @Translation("Alerta"),
 *   description = @Translation("Provides the Alerta channel plugin.")
 * )
 */
class Alerta extends ChannelBase {

  /**
   * {@inheritdoc}
   */
  public function getConfigName(): string {
    return 'pf_alerta.settings';
  }

  /**
   * {@inheritdoc}
   */
  public function applicable(UserInterface $user): bool {
    // This channel sends directly, so it never applies to other notifications.
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function send(UserInterface $user, ContentEntityInterface $entity, array $content, int $attempt): string {
    $body = [
      'origin' => $this->pluginConfig->get('host'),
      'timeout' => 86400,
      'environment' => $this->pluginConfig->get('environment'),
      'resource' => 'undefined-' . Drupal::time()->getRequestTime(),
      'event' => 'Exception',
      'tags' => ['environment:' . $this->pluginConfig->get('environment')],
      'group' => 'Application',
      'severity' => 'critical',
      'service' => ['www.example.com'],
      'text' => '',
      'value' => '',
      'rawData' => '',
    ];
    if (!empty($this->pluginConfig->get('pid'))) {
      $body['attributes'] = [
        'gitlab_project_id' => $this->pluginConfig->get('pid'),
      ];
    }

    // TODO: Use pre-rendered content instead of doing it again.
    if (method_exists($entity, 'getPayload')) {
      $payload = $entity->getPayload();
      if (method_exists($payload, 'formatAlerta')) {
        $payload->formatAlerta($body);
      }
    }

    $options = [
      'headers' => [
        'Authorization' => 'Key ' . $this->pluginConfig->get('apikey'),
        'Content-Type' => 'application/json',
      ],
      'body' => json_encode($body),
    ];

    $client = new Client(['base_uri' => $this->pluginConfig->get('url') . '/alert']);
    try {
      $response = $client->request('POST', NULL, $options);
      $statusCode = $response->getStatusCode();
      if ($statusCode === 201) {
        return self::RESULT_STATUS_SUCCESS;
      }
    }
    catch (GuzzleException $e) {
      // TODO: Log this exception.
    }
    return self::RESULT_STATUS_FAILED;
  }

}
